#!/opt/VRTSsfmh/bin/perl
# $Id: array_driver.pl,v 1.11 2014/05/27 06:52:00 myadav Exp $
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }

use strict;
use Getopt::Long;
use File::Basename;
use Fcntl ':flock';

# set the lib dir appropriatly
my $bdir = dirname("$0");
$bdir=$bdir."/../lib/modules";
push @INC, $bdir;

$bdir = dirname("$0");
$bdir=$bdir."/../agentlet/";
push @INC, $bdir;

require VRTS::Agent;
require VRTS::Util;
require VRTS::EXITCODE;
require VRTS::Paths;
require VRTS::Logger;
require VRTS::OS;
require File::Path;
require VRTS::Config;
require VRTS::Task;
require VRTS::Enclosure::Capacities;
require VRTS::Enclosure::Stats;
require VRTS::Enclosure::Utils;

umask 027;
my $driverbasedir;
my $drivervardir;
$driverbasedir=VRTS::Paths::get_path("InstallDir");
$ENV{"DRIVERBASEDIR"} = "$driverbasedir";
$drivervardir=VRTS::Paths::get_path("AppDir");
$ENV{"DRIVERVARDIR"} = "$drivervardir";

sub print_error
{
  print "$0 : invalid options\n";
  exit(&VRTS::EXITCODE::USAGE_ERROR);
}

sub usage
{
  print "$0 --family <familyname> [--discover] --hidden \n";
  print "   family:   Name of a family that needs to be run\n";
  print "   discover:   Run full scale discovery for specific family\n";
  print "   wait:   Ensure that the discovery wait for the current run to complete and then follow it by another run\n";
  exit(&VRTS::EXITCODE::USAGE_ERROR);
}

my %opts;

my $ret = GetOptions(\%opts,qw (family=s discover h help hidden wait context=s report-task));
my $family;
my $reportstyle;
my $basedir;
my $pwdir = dirname("$0");
my $CONFIG_FILE = $pwdir."/../lib/modules/VRTS/Config.pm";
my $BASEVERSION_TAG = "MANIFEST_BASEVERSION";
my $VERSION_TAG = "MANIFEST_VERSION";
my $hostid;
my $cmd;
my $bin_path = VRTS::Paths::get_path("MHBin");


my %context = ();
if ($opts{"context"} ne "")
{
    my $spt_pattr = qr/:|=/;
    %context = split(/$spt_pattr/, $opts{"context"});
}

if($ret eq "")
{
    usage();
}

if($opts{"h"} ne "" or $opts{"help"} ne "")
{
    usage();
}

if($opts{"hidden"} eq "")
{
    print_error();
}

if( $opts{"family"} ne "")
{
    $family = $opts{"family"};
}
else
{
    usage();
}


# Bail if this is invoked on the slave node.
if (VRTS::Util::is_ha_slave()) 
{
    VRTS::Logger::log('error', "Host is a HA slave node, $family family should not run on a slave");
    exit 0;
}

$reportstyle = "JSONReporter";
$ENV{"REPORTSTYLE"}="$reportstyle";

my $familybasedir = "$driverbasedir/agentlet/VRTS/$family";
if ($^O =~ /Win32/i)
{
    $familybasedir = "$driverbasedir"."agentlet\\VRTS\\$family";
}
$ENV{"FAMILYBASEDIR"} = "$familybasedir";

push(@INC, "$driverbasedir/agentlet");

my $familyvardir = $drivervardir.$family;
$ENV{"FAMILYVARDIR"} = "$familyvardir";

my ($lockfile, $con_lock);
my $wait_familylock;
if (not defined $context{vardir} or $context{vardir} eq "")
{
    $lockfile = $familyvardir."/$family.lock";
    $wait_familylock = $familyvardir."/$family"."_wait.lock";
}
else
{
    my $f_name = $family.".".$context{vardir};
    $lockfile = $familyvardir.".".$context{vardir}."/".$family.".lock";
    $con_lock = $familyvardir."/$family.lock";
    $wait_familylock = $familyvardir."/$family"."_wait.lock";
}


if(! -d $familyvardir)
{
    if (VRTS::Util::is_ha_slave()) 
    {
        # On slaves nodes if the shared directory for 
        # family var dir is not accessible then terminate discovery
        exit 0;
    }
    eval ( File::Path::mkpath("$familyvardir", 0, 0755) );
    if($@)
    {
      VRTS::Logger::log('error', "Cannot mkdir $@");
      exit(&VRTS::EXITCODE::MKDIR_ERROR);
    }
}

my $ensure =  $drivervardir."enrescan.lock";

if (defined $context{vardir} and $context{vardir} ne "")
{
    my $dir_name = $familyvardir."/".$context{vardir};
    $lockfile = $familyvardir."/".$context{vardir}."/".$family.".lock";

    if (! -d $dir_name)
    {
        eval ( File::Path::mkpath("$dir_name", 0, 0755) );
        if($@)
        {
            log_before_exit("Cannot mkdir $dir_name:[$@]");
            exit(&VRTS::EXITCODE::MKDIR_ERROR);
        }
    }
}
else
{
    exit(&VRTS::EXITCODE::MKDIR_ERROR);
}


# check if cmsf_driver is already running for this family
unless (open  FH, ">$lockfile")
{
    VRTS::Logger::log('error', "Cannot open $@");
    exit(&VRTS::EXITCODE::MKDIR_ERROR);
}
else
{
    if (defined $opts{"wait"} and $opts{"wait"} ne "")
    {
        my $got_wait_lock = 0;

        unless (open  WFH, "> $wait_familylock")
        {
            log_before_exit("Cannot open $@");
            die "Cannot open $@";

        }

        if (!flock (WFH, LOCK_EX|LOCK_NB))
        {
            $got_wait_lock = 0;
            flock (WFH, LOCK_EX);
        }
        else
        {
            $got_wait_lock = 1;
        }

        # Now take a blocking lock in this case, as we want to wait.
        flock (FH, LOCK_EX);

        # Release the wait lock as the discovery for the wait run just started.
        close(WFH);

        exit 0 if ($got_wait_lock == 0);
    }
    else
    {
        if (!flock (FH, LOCK_EX|LOCK_NB))
        { 
            if (($context{RESCAN} == 1) && ($context{ENSURE} == 1))
            {
                # A trigger could be running when a rescan
                # is called. during rescan if the same family
                # gets triggered before its previous run is 
                # complete then rescan bails out. ensure it 
                # during the next trigger. 

                open (E, ">$ensure");
                close (E);  

            }
            print STDERR "$0 is already running for $family :: $context{vardir} \n";
            exit (1);
        }
    }
}

# Make the blocking wait here, this will have ensured that arrays whose
# discovery is still in progress is removed from waiting in this queue.
#
open(CON_LOCK, "> $con_lock");
flock (CON_LOCK, LOCK_EX);

my $schedule_id = $context{vardir};
my $taskobj = new VRTS::Task();
if (defined $taskobj)
{
    my $ret = $taskobj->load (undef, $schedule_id);
    if ($ret == 0)
    {
        $taskobj->update ($schedule_id, {Name => "$family [$schedule_id]",
                                         state => &VRTS::Task::TASK_STATE_RUNNING, 
                                         StartTime =>""});
    }
    else
    {
        my $tinfo = {
                        taskid => $schedule_id,
                        Name => "$family [$schedule_id]",
                        category => &VRTS::Task::TASK_CATEGORY_UNPUBLISHED
                    };

        $taskobj->create ("discover.family.array.$family", $tinfo);
    }
}

my $configure_taskid;

if ($opts{"report-task"} ne "")
{
    my $taskinfo =
    {
        Name => "Add Enclosure",
        TargetType => "vrts_ddl_enclosure_config",
        Target => $context{vardir},
        RptOnStChnge => 1
    };

    $configure_taskid = $taskobj->create("enclosure.config", $taskinfo);
}


my $agent = new VRTS::Agent;
$agent->initialize($family, $hostid, \%context);

if($opts{"discover"} ne "")
{
   $agent->{CONFIGOPTIONS}->{RESCAN} = 1;
}
else
{
   $agent->{CONFIGOPTIONS}->{RESCAN} = 0;
}

if (!defined $context{'execution-ctxt'} || $context{'execution-ctxt'} ne "sdktest")
{
    VRTS::Hook::register("PreDiscovery",
           "VRTS::Enclosure::Utils::check_config_health");
}

VRTS::Hook::register("PreDiff",
           "VRTS::Enclosure::Discover::update_config_mapping");

VRTS::Hook::register("PreDiff", 
           "VRTS::Enclosure::Capacities::compute_capacities");

VRTS::Hook::register("PreDiff",
           "VRTS::Enclosure::Capacities::report_capabilities");

VRTS::Hook::register("PreDiff", "VRTS::Enclosure::Stats::store_stats");

VRTS::Hook::register("PostReport",
           "VRTS::Enclosure::Utils::setup_stat_cfg");

eval {$ret = $agent->run()};
if ($ret)
{
    VRTS::Logger::log('error', "Discovery Run failed: $@");
}

#Terminate the task after complete discovery procedure is done.
#This will be applicable only in case of refresh of enclosure 
#where we pass taskid in context.
VRTS::Enclosure::Utils::terminate_task($agent);

if (defined $schedule_id)
{
    # Set task to complete. Set nonzero Result if error
    my $result = 0;
    $result = 1 if ($ret != 0);

    my @output;
    if ($result)
    {
        push @output, $@;
    }
    $taskobj->update ($schedule_id, {state=> &VRTS::Task::TASK_STATE_FINISHED, 
                                     Result=>$ret, outputs => \@output});
}

# Terminate the refresh task.
if ($opts{"report-task"} ne "")
{
     # Try and obtain the name of the enclosure to fix task name.
     my $encl_name = VRTS::Enclosure::Utils::get_encl_name_using_vendorid($context{vardir});

     if (defined $encl_name and $encl_name ne "")
     {
          $taskobj->update_field($configure_taskid, 'Target', $encl_name);
     } 
     $taskobj->update_field($configure_taskid, 'state', "FINISHED");
}

close(CON_LOCK);

exit($ret);
