#$Id: Correlation.pm,v 1.1 2012/09/19 05:54:50 vkaul Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Correlation;
use strict;
use Carp;
require Exporter;
use File::Spec;
use Data::Dumper;
use File::Basename;
use Fcntl ':flock';
use FileHandle;

require VRTS::Paths;
require JSON;
require VRTS::Util;
require VRTS::Summary;
require VRTS::Enclosure::Utils;

require VRTS::Constants::P_ARRAY_NAS_SHARE;
require VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT;
require VRTS::Constants::P_ARRAY_SUBSYSTEM;

use vars qw($VERSION @ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(correlate_nas_share_esx_sp pull_esx_storagepool_data);
$VERSION = '3.0';

use constant UPDATE_SP_CACHE => 7200; # once every 2 hrs.
my @hid;
sub correlate_nas_share_esx_sp
{
    my $agent = shift;
    my $ip_list = shift;

    my $ret = 0;
    my $pool_hash = undef;

    ($ret, $pool_hash) = pull_esx_storagepool_data($agent);

    return 1 if ($ret != 0 or not defined $pool_hash);

    # Seems like we have all the data is required to perform corelation.

    my $nfs_export_tb = $agent->gettable("P_ARRAY_NAS_SHARE");
    my $nfs_rows =
          $nfs_export_tb->getallrowswithvalue(
                  &VRTS::Constants::P_ARRAY_NAS_SHARE::MOUNT_POINT_TYPE, "NFS");

    return if (not defined $nfs_rows);

    my $dev_assg_tb = $agent->gettable("P_ARRAY_DEVICE_ASSIGNMENT");
    my $enc_tb = $agent->gettable("P_ARRAY_SUBSYSTEM");

    foreach my $nfs_r (@{$nfs_rows})
    {
        # Try and construct the possible url that is obtained from the ESX discovery side.
        foreach my $tmp_ip (@{$ip_list})
        {
            my $pool_id = "netfs://$tmp_ip/$nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::PATH]/";
            my $atl_pool_id = "netfs://$tmp_ip"."$nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::PATH]/";

            if (defined $pool_hash->{$pool_id})
            {
                # For the specific nas share, get parent/container information as well.
                my $enc_row = $enc_tb->getrowwithvalue(&VRTS::Constants::P_ARRAY_SUBSYSTEM::ID,
                                              $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::SUBSYS_ID]);
 
                _report_device_assignment($agent, $dev_assg_tb,
                      $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::ID],
                      "NFS_SHARE",
                      $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME],
                      $pool_hash->{$pool_id}->{id},
                      "STORAGE_POOL",
                      $pool_hash->{$pool_id}->{name},
                      $enc_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ID],
                      "ENCLOSURE",
                      $enc_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::NAME]);

                last;
            }
            elsif (defined $pool_hash->{$atl_pool_id})
            {
                # For the specific nas share, get parent/container information as well.
                my $enc_row = $enc_tb->getrowwithvalue(&VRTS::Constants::P_ARRAY_SUBSYSTEM::ID,
                                              $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::SUBSYS_ID]);

                _report_device_assignment($agent, $dev_assg_tb,
                      $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::ID],
                      "NFS_SHARE",
                      $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME],
                      $pool_hash->{$atl_pool_id}->{id},
                      "STORAGE_POOL",
                      $pool_hash->{$atl_pool_id}->{name},
                      $enc_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ID],
                      "ENCLOSURE",
                      $enc_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::NAME]);

                last;
            }
        }
    }
}

sub pull_esx_storagepool_data
{
    my $agent = shift;

    my $context = $agent->get_context();

    my $app_dir = VRTS::Paths::get_path("AppDir");
    $app_dir = File::Spec->catdir($app_dir, "SIData");

    eval { File::Path::mkpath($app_dir, 0, 755); };

    return (1, undef) if (! -d $app_dir);

    my $last_time = undef;
    my $time_f = File::Spec->catfile($app_dir, ".cache_status");
    my $cache_file = File::Spec->catfile($app_dir, "storage_pool.cache");
    my $cache_lock = File::Spec->catfile($app_dir, "storage_pool.lock");

    if (-f $time_f)
    {
        open (HAN, $time_f);
        $last_time = <HAN>;
        close(HAN);

        chomp $last_time;
    }
    else
    {
        $last_time = 0; # this will force the cache to be built up for the very first time.
    }

    my $lc_time = time;
    # Compare this with what is stampped in the file.

    # Acquire a lock and a blocking lock in this case.
    open(CACHE_LOCK, "> $cache_lock");
    flock(CACHE_LOCK, LOCK_EX);

    my ($ret, $op);
    if ($lc_time > ($last_time + &UPDATE_SP_CACHE) or $context->{update_cache} == 1)
    {
        VRTS::Logger::log('debug', "Need to refresh the cache under [$cache_file]");

        # Lets get the NFS type storage pools from the database.
        my $recs = VRTS::Enclosure::Utils::pull_nfs_storage_pools();
        return (1, undef) if (not defined $recs);

        my $pool_hash = _massage_pool_data($recs);
        my $ret = _sync_cache($pool_hash, $cache_file);

        if ($ret != 0)
        {
            $ret = 1;
            $op = undef;
            VRTS::Logger::log('debug', "Nothing obtained to generate cache, skip correlation for now");
            goto SKIP_MARKER_CREATION;
        }
        else
        {
            $ret = 0;
            $op = $pool_hash;
        }

        open(HAN, "> $time_f");
        print HAN $lc_time;
        close(HAN);
    }
    else
    {
        VRTS::Logger::log('debug', "No need to refresh the cache, reading cache from [$cache_file]");
        ($ret, $op) = _read_cache($cache_file);

        if ($ret == 2)
        {
            VRTS::Logger::log('debug', "Invalidating the cache and its contents, along with timestamp file");
            unlink ("$time_f");
            unlink ("$cache_file");

            my @ignore_tbs = ("P_ARRAY_DEVICE_ASSIGNMENT");
            $agent->ignoretables(\@ignore_tbs);
        }
    }

SKIP_MARKER_CREATION:

    # Release the lock in this case.
    close(CACHE_LOCK);

    return $ret, $op;
}

# Some helper routines to manage a cache.

sub _report_device_assignment
{
    my $agent = shift;
    my $map_tb = shift;
    my $source_id = shift;
    my $source_type = shift;
    my $source_name = shift;
    my $dest_id = shift;
    my $dest_type = shift;
    my $dest_name = shift;
    my $source_con_id = shift;
    my $source_con_type = shift;
    my $source_con_name = shift;
    my $dest_con_id = shift;
    my $dest_con_type = shift;
    my $dest_con_name = shift;


    my @map_row;

    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_ID] = $source_id;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_TYPE] = $source_type;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_NAME] = $source_name;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_ID] = $dest_id;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_TYPE] = $dest_type;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_NAME] = $dest_name;

    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_CONTAINER_ID] = $source_con_id;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_CONTAINER_TYPE] = $source_con_type;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_CONTAINER_NAME] = $source_con_name;

    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_CONTAINER_ID] = $dest_con_id;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_CONTAINER_TYPE] = $dest_con_type;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_CONTAINER_NAME] = $dest_con_name;

    $map_tb->addrow(\@map_row);
    #$map_tb->print_table();
}

sub _report_host_device_assignment
{
    my $agent = shift;
    my $map_tb = shift;
    my $source_id = shift;
    my $source_type = shift;
    my $source_name = shift;
	my $pool_hash = shift;
	my $share_id = shift;
	my $e_vdid = shift;
	my $enc_name = shift;
	my $enc_type = shift;
    my @map_row;

    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_ID] = $source_id;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_TYPE] = $source_type;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_NAME] = $source_name;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_ID] =
	    $pool_hash->{$share_id}->{dest_id};
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_TYPE] = 
	    $pool_hash->{$share_id}->{dest_type};
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_NAME] = 
	    $pool_hash->{$share_id}->{dest_name};
	$map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_CONTAINER_ID] =
	    $e_vdid;
    $map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_CONTAINER_TYPE] =
	    $enc_type;
	$map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::SOURCE_CONTAINER_NAME]
	    = $enc_name;
	$map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_CONTAINER_ID] = 
	    $pool_hash->{$share_id}->{dest_container_id};
	$map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_CONTAINER_NAME] =
	    $pool_hash->{$share_id}->{dest_container_name};
	$map_row[&VRTS::Constants::P_ARRAY_DEVICE_ASSIGNMENT::DEST_CONTAINER_TYPE] =
	    $pool_hash->{$share_id}->{dest_container_type};

    $map_tb->addrow(\@map_row);
    my $share_tb = $agent->gettable("P_ARRAY_SHARE_CONSUMER");
    my $nfs_rows =
         $share_tb->getallrowswithvalue(
                  &VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_NAME, $pool_hash->{$share_id}->{dest_container_name});
    foreach my $nfs_r (@{$nfs_rows})
    {
        $nfs_r->[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_ID] = $pool_hash->{$share_id}->{dest_container_id};
    }
    # Check if mapped using IP
    $nfs_rows = $share_tb->getallrowswithvalue(
                   &VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_NAME, $pool_hash->{$share_id}->{dest_container_ip});
    foreach my $nfs_r (@{$nfs_rows})
    {
        $nfs_r->[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_ID] = $pool_hash->{$share_id}->{dest_container_id};
    }
    # For CFS shares
    $share_id =~ /^\\(\d+\.\d+\.\d+\.\d+)\\/;
    $nfs_rows = $share_tb->getallrowswithvalue(
                &VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_NAME, $1);
    foreach my $nfs_r (@{$nfs_rows})
    {
        $nfs_r->[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_ID] = $pool_hash->{$share_id}->{dest_container_id};
    }
}

sub _read_cache
{
    my $cache_file = shift;

    return 1, undef if (! -f $cache_file);

    open FILE, "< $cache_file";
    my $buf = do { local $/; <FILE> };
    close(FILE);

    my $jsonobject = new JSON ();
    my $data = undef;

    eval {
        $data = $jsonobject->jsonToObj($buf);
    };

    if ($@)
    {
        VRTS::Logger::log('error', "Cache seems invalid, need to re-generate");
        return 2, undef;
    }

    return 0, $data;
}

sub _sync_cache
{
    my $hash = shift;
    my $cache_file = shift;

    my @key = (keys %{$hash});

    return 1 if (not defined @key or scalar(@key) == 0);

    my $final_buf = JSON::objToJson($hash, {pretty => 1, indent => 1});

    my $fd = new FileHandle $cache_file, O_WRONLY|O_CREAT;
    print $fd "$final_buf\n";

    close($fd);

    return 0;
}

sub _massage_pool_data
{
    my $recs = shift;

    my %pool_hash = ();

    foreach my $r (@{$recs})
    {
        $pool_hash{$r->{POOL_UUID}}->{id} = $r->{ID};
        $pool_hash{$r->{POOL_UUID}}->{name} = $r->{NAME};
    }

    return \%pool_hash;
}


sub _massage_share_data
{
    my $recs = shift;

    my %pool_hash = ();

    foreach my $r (@{$recs})
    {
        $r->{ID} =~ /(.*)@(.*)/;
        push(@hid, $2);
        $pool_hash{$r->{ID}}->{dest_id} = $r->{ID};
        $pool_hash{$r->{ID}}->{dest_name} = $r->{SHARE_ID};
        $pool_hash{$r->{ID}}->{dest_type} = 'FILESYSTEM';
        $pool_hash{$r->{ID}}->{dest_container_id} = $r->{HOST_ID};
        $pool_hash{$r->{ID}}->{dest_container_name} = $r->{NAME};
        $pool_hash{$r->{ID}}->{dest_container_type} = 'HOST';
        $pool_hash{$r->{ID}}->{dest_container_ip} = $r->{IP};
    }

    return \%pool_hash;
}

sub correlate_nas_share_host
{
    my $agent = shift;
    my $ip_list = shift;
	my $e_vdid = shift;
	my $enc_name = shift;
	my $enc_type = shift;

    my $ret = 0;
    my $pool_hash = undef;

    ($ret, $pool_hash) = pull_host_share_data($agent);

    return 1 if ($ret != 0 or not defined $pool_hash);

    # Seems like we have all the data is required to perform corelation.

    my $nfs_export_tb = $agent->gettable("P_ARRAY_NAS_SHARE");
    my $nfs_rows =
          $nfs_export_tb->getallrowswithvalue(
                  &VRTS::Constants::P_ARRAY_NAS_SHARE::MOUNT_POINT_TYPE, "NFS");

    return if (not defined $nfs_rows);

    my $dev_assg_tb = $agent->gettable("P_ARRAY_DEVICE_ASSIGNMENT");
    foreach my $nfs_r (@{$nfs_rows})
    {
      # Try and construct the possible url that is obtained from the Host discovery side.
      foreach my $host (@hid) {
        foreach my $tmp_ip (@{$ip_list})
        {
            my $share_id = "$tmp_ip:$nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::PATH]" . "@".$host;
            if (defined $pool_hash->{$share_id})
            {
                _report_host_device_assignment($agent, $dev_assg_tb,
                      $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::ID],
                      "NFS_SHARE",
                      $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME],
					  $pool_hash, $share_id, $e_vdid, $enc_name, $enc_type);
                last;
            }
        }
        # This is to handle AIX case or any other case where the share_id
        # matches format /share:@hostid. Ex:
        # "/APPS_POOL_NAS_FS@{00090011-25c5-7758-0000-00000ad159a8}"
        my $share_id = $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::PATH] .
                       "@".$host;
        if (defined $pool_hash->{$share_id})
        {
            _report_host_device_assignment($agent, $dev_assg_tb,
                    $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::ID],
                    "NFS_SHARE",
                    $nfs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME],
                    $pool_hash, $share_id, $e_vdid, $enc_name, $enc_type);
        }
      }
    }
    my $cifs_rows = $nfs_export_tb->getallrowswithvalue(
		    &VRTS::Constants::P_ARRAY_NAS_SHARE::MOUNT_POINT_TYPE, "CIFS");
    return if (not defined $cifs_rows);
    foreach my $cifs_r (@{$cifs_rows})
    {
        # Try and construct the possible url that is obtained from the Host discovery side.
	foreach my $host (@hid) {
	   foreach my $tmp_ip (@{$ip_list})
	   {
	       my $share_id = "\\$tmp_ip\\$cifs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME]" . "@".$host;
	       if (defined $pool_hash->{$share_id})
	       {
		   _report_host_device_assignment($agent, $dev_assg_tb,
			    $cifs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::ID],
			    "NFS_SHARE",
			    $cifs_r->[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME],
			    $pool_hash, $share_id, $e_vdid, $enc_name, $enc_type);
			    last;
               }
            }
        }
    }
}

sub pull_host_share_data
{
    my $agent = shift;

    my $context = $agent->get_context();

    my $app_dir = VRTS::Paths::get_path("AppDir");
    $app_dir = File::Spec->catdir($app_dir, "SIData");

    eval { File::Path::mkpath($app_dir, 0, 755); };

    return (1, undef) if (! -d $app_dir);

    my $last_time = undef;
    my $time_f = File::Spec->catfile($app_dir, ".cache_status_host");
    my $cache_file = File::Spec->catfile($app_dir, "host_share.cache");
    my $cache_lock = File::Spec->catfile($app_dir, "host_share.lock");

    if (-f $time_f)
    {
        open (HAN, $time_f);
        $last_time = <HAN>;
        close(HAN);

        chomp $last_time;
    }
    else
    {
        $last_time = 0; # this will force the cache to be built up for the very first time.
    }

    my $lc_time = time;
    # Compare this with what is stampped in the file.

    # Acquire a lock and a blocking lock in this case.
    open(CACHE_LOCK, "> $cache_lock");
    flock(CACHE_LOCK, LOCK_EX);

    my ($ret, $op);
    if ($lc_time > ($last_time + &UPDATE_SP_CACHE) or $context->{update_cache} == 1)
    {
        VRTS::Logger::log('debug', "Need to refresh the cache under [$cache_file]");

        # Lets get the NFS from the database.
        my $recs = VRTS::Enclosure::Utils::pull_nfs_shares();
        return (1, undef) if (not defined $recs);

        my $pool_hash = _massage_share_data($recs);
        my $ret = _sync_cache($pool_hash, $cache_file);

        if ($ret != 0)
        {
            $ret = 1;
            $op = undef;
            VRTS::Logger::log('debug', "Nothing obtained to generate cache, skip correlation for now");
            goto SKIP_MARKER_CREATION;
        }
        else
        {
            $ret = 0;
            $op = $pool_hash;
        }

        open(HAN, "> $time_f");
        print HAN $lc_time;
        close(HAN);
    }
    else
    {
        # Lets get the NFS from the database.
        my $recs = VRTS::Enclosure::Utils::pull_nfs_shares();
        return (1, undef) if (not defined $recs);

        my $pool_hash = _massage_share_data($recs);
        VRTS::Logger::log('debug', "No need to refresh the cache, reading cache from [$cache_file]");
        ($ret, $op) = _read_cache($cache_file);

        if ($ret == 2)
        {
            VRTS::Logger::log('debug', "Invalidating the cache and its contents, along with timestamp file");
            unlink ("$time_f");
            unlink ("$cache_file");

            my @ignore_tbs = ("P_ARRAY_DEVICE_ASSIGNMENT");
            $agent->ignoretables(\@ignore_tbs);
        }
    }

SKIP_MARKER_CREATION:
    # Release the lock in this case.
    close(CACHE_LOCK);

    return $ret, $op;
}


1;
