#$Id: Discover.pm,v 1.15 2014/05/09 12:54:13 pkeshvan Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover;
use strict;
use Carp;
require Exporter;
require VRTS::Enclosure::Utils;

require VRTS::Constants::P_ARRAY_CONFIG;
require VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING;
require VRTS::Constants::P_ARRAY_SUBSYSTEM;


use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

@ISA = qw(Exporter);
@EXPORT = qw(get_enclosure_listing refresh_device_manager);
$VERSION = '3.0';

use constant PROBE_SUCCESS => "0"; # Probe is a sucess, there is no problem found.
use constant VENDOR_COMP_NOT_FOUND => "1001"; # The required vendor components is not installed on the control host.
use constant VENDOR_COMP_VERSION_INCOMPAT => "1002"; # Vendor component present but not the required version for us to discover the array..
use constant FMW_VERSION_INCOMPAT => "1003"; # Vendor component is there, required version is there but the array side firmware version is not supported..
use constant NO_ARRAYS_DISCOVERED => "1004"; #All required components are present with required criteria, but there are no devices that are present..
use constant PROBE_INTERNAL_ERROR => "1010"; # there is some internal array when performing probe array.
use constant MH_NOT_REACHABLE => "1005"; # push exec called and we did not get response from the managed host.
use constant CONNECT_FAILED => "1011"; # primary as well as secondary CH failed to contact HiCommand
use constant SEC_CONNECT_FAILED=> "1012"; # primary working but secondary CH failed to contact HiCommand
use constant VERIFICATION_FAILED => "1013"; # refresh symmetrix device returned back that the sym db and array are not in sync.
use constant INCORRECT_PASSWORD => "1014"; # Incorrect password provided
use constant USER_NAME_DOES_NOT_EXIST=> "1015"; # user does not exist
use constant NO_CONNECTION => "1016"; # IP address is invalid
use constant ARRAY_NOT_REACHABLE => "1017"; # Array is not reachable
use constant AUTHENTICATION_FAILED => "1018"; #Authentication failed.Communication with array failed with given inputs. Please verify and re-enter
use constant ARRAY_AUTHORIZATION_FAILED => "1019"; #Array credentials or IP or Port supplied is incorrect
use constant PLATFORM_UNSUPPORTED => "1020"; #Array discovery from the specified host platform is unsupported
use constant SECURITY_ROLE_NOT_PRIVILEGE => "1021"; # User does not have sufficient previleges to get the data.
use constant SYMAPI_SERVER_NOT_VALID => "1022"; # The symapi server mentioned is invalid
use constant HPEVA_HTTPS_ERROR_CONN => "1023"; # hpeva CV is down, or incorrect IP or incorrect username/password
use constant HPEVA_SSSU_VERSION_INCOMPAT => "1024"; # hpeva sssu version is lower than 8.0. Not supported
use constant SESSION_TIMEDOUT => "1025"; #Connection session timedout
use constant NO_CONNECTION_INVALID_IP => "1028"; # IP address entered is invalid or unreachable
use constant DSCLI_PASSWD_PASSWDFILE_NOT_DEFINED => "1030"; # Please enter either password or password file location
use constant DSCLI_PASSWDFILE_NOT_FOUND => "1031"; # Password File not found at entered location
use constant DSCLI_PASSWDFILE_RECORD_NOT_FOUND => "1032"; # Storage Array IP - User Name record does not exist in password file
use constant HPEVA_SSSU_PASSWD_PASSWDFILE_NOT_DEFINED => "1033"; # Please enter either password or password file location
use constant HPEVA_SSSU_PASSWDFILE_NOT_FOUND => "1034"; # Password File not found at entered location
use constant HPEVA_SSSU_CV_MANAGER_NOT_DEFINED_IN_PASSWDFILE => "1035"; #HPEVA -Password is not specified and the password-file does not have the CV Manager entry
use constant SVC_WRONG_USERNAME_AND_WRONG_SSHFILE => "1037"; # Please enter correct combination of username and ssh file
use constant SVC_WRONG_USERNAME_FOR_SSHFILE => "1038"; # Please enter correct username for the specified ssh file
use constant SVC_MH_VERSION_NOT_SUPPORTED => "1039"; # Please enter discovery host with MH version 6.0 or above
use constant SVC_WRONG_CREDENTIALS_OR_WRONG_USERNAME_AND_WRONG_SSHFILE => "1040"; # Please enter correct credentials or correct combination of username and ssh file
use constant ENCLOSURE_PASSWD_EXPIRED => "1041";
use constant ARRAY_DISCOVERY_FM_LOWER_VERSION_ERROR => "1055";
use constant ENCLOSURE_CERT_CN_MISMATCH => "1059";
use constant XML_API_SERVER_NOT_RESPONDING => "1057"; # XML API Server on Control Station either not running or not responding
use constant VNX_BOTH_ENCLOSURE_FIELDS_EMPTY => 1058; # Both Control Station and Block IP fields cannot be empty at the same time


my %err_resource_map = (
   &PROBE_SUCCESS => "PROBE_SUCCESS",
   &VENDOR_COMP_NOT_FOUND => "VENDOR_COMP_NOT_FOUND",
   &VENDOR_COMP_VERSION_INCOMPAT => "VENDOR_COMP_VERSION_INCOMPAT",
   &FMW_VERSION_INCOMPAT => "FMW_VERSION_INCOMPAT",
   &NO_ARRAYS_DISCOVERED => "NO_ARRAYS_DISCOVERED",
   &PROBE_INTERNAL_ERROR => "PROBE_INTERNAL_ERROR",
   &MH_NOT_REACHABLE => "MH_NOT_REACHABLE",
   &CONNECT_FAILED => "CONNECT_FAILED",
   &SEC_CONNECT_FAILED => "SEC_CONNECT_FAILED",
   &VERIFICATION_FAILED => "VERIFICATION_FAILED",
   &INCORRECT_PASSWORD => "INCORRECT_PASSWORD",
   &USER_NAME_DOES_NOT_EXIST => "USER_NAME_DOES_NOT_EXIST",
   &NO_CONNECTION => "NO_CONNECTION",
   &ARRAY_NOT_REACHABLE => "ARRAY_NOT_REACHABLE",
   &AUTHENTICATION_FAILED => "AUTHENTICATION_FAILED",
   &ARRAY_AUTHORIZATION_FAILED => "ARRAY_AUTHORIZATION_FAILED",
   &PLATFORM_UNSUPPORTED => "PLATFORM_UNSUPPORTED",
   &SECURITY_ROLE_NOT_PRIVILEGE => "SECURITY_ROLE_NOT_PRIVILEGE",
   &SYMAPI_SERVER_NOT_VALID => "SYMAPI_SERVER_NOT_VALID",
   &HPEVA_HTTPS_ERROR_CONN => "HPEVA_HTTPS_ERROR_CONN",
   &HPEVA_SSSU_VERSION_INCOMPAT => "HPEVA_SSSU_VERSION_INCOMPAT",
   &SESSION_TIMEDOUT => "SESSION_TIMEDOUT",
   &NO_CONNECTION_INVALID_IP => "NO_CONNECTION_INVALID_IP",
   &DSCLI_PASSWD_PASSWDFILE_NOT_DEFINED => "DSCLI_PASSWD_PASSWDFILE_NOT_DEFINED",
   &DSCLI_PASSWDFILE_NOT_FOUND=> "DSCLI_PASSWDFILE_NOT_FOUND",
   &DSCLI_PASSWDFILE_RECORD_NOT_FOUND=> "DSCLI_PASSWDFILE_RECORD_NOT_FOUND",
   &ENCLOSURE_PASSWD_EXPIRED=> "ENCLOSURE_PASSWD_EXPIRED",
   &HPEVA_SSSU_PASSWD_PASSWDFILE_NOT_DEFINED => "HPEVA_SSSU_PASSWD_PASSWDFILE_NOT_DEFINED",
   &HPEVA_SSSU_PASSWDFILE_NOT_FOUND => "HPEVA_SSSU_PASSWDFILE_NOT_FOUND",
   &HPEVA_SSSU_CV_MANAGER_NOT_DEFINED_IN_PASSWDFILE => "HPEVA_SSSU_CV_MANAGER_NOT_DEFINED_IN_PASSWDFILE",
   &SVC_WRONG_USERNAME_AND_WRONG_SSHFILE => "SVC_WRONG_USERNAME_AND_WRONG_SSHFILE",
   &SVC_WRONG_USERNAME_FOR_SSHFILE => "SVC_WRONG_USERNAME_FOR_SSHFILE",
   &SVC_MH_VERSION_NOT_SUPPORTED => "SVC_MH_VERSION_NOT_SUPPORTED",
   &SVC_WRONG_CREDENTIALS_OR_WRONG_USERNAME_AND_WRONG_SSHFILE => "SVC_WRONG_CREDENTIALS_OR_WRONG_USERNAME_AND_WRONG_SSHFILE",
   &ARRAY_DISCOVERY_FM_LOWER_VERSION_ERROR => "ARRAY_DISCOVERY_FM_LOWER_VERSION_ERROR",
   &ENCLOSURE_CERT_CN_MISMATCH => 'ENCLOSURE_CERT_CN_MISMATCH',
   &XML_API_SERVER_NOT_RESPONDING => "XML_API_SERVER_NOT_RESPONDING",
   &VNX_BOTH_ENCLOSURE_FIELDS_EMPTY => 'VNX_BOTH_ENCLOSURE_FIELDS_EMPTY'
);

# Some hash defines for replication states.
use constant NOT_REPLICATED => 0;
use constant LOCAL_REPLICATION => 1;
use constant REMOTE_REPLICATION_PRIMARY => 2;
use constant REMOTE_REPLICATION_SECONDARY => 3;
use constant LOCAL_REPLICATION_REMOTE_REPLICATION_PRIMARY => 4;
use constant LOCAL_REPLICATION_REMOTE_REPLICATION_SECONDARY => 5;

# Some constants to be used as part of initial discovery as well as refresh
use constant DISCOVERY_FINISHED => 0;
use constant DISCOVERY_IN_PROGRESS => 1;
use constant DISCOVERY_FAILED => 2;

my %msg_map = (&PROBE_SUCCESS => "Probe operation is a success",
               &VENDOR_COMP_NOT_FOUND => "The required vendor components not found at the specified location",
               &VENDOR_COMP_VERSION_INCOMPAT => "Required vendor component version not installed",
               &FMW_VERSION_INCOMPAT => "Minimum firmware version not found for enclosure discovery",
               &NO_ARRAYS_DISCOVERED => "No enclosure(s) discovered via the specified discovery mechanism",
               &PROBE_INTERNAL_ERROR => "Internal error encountered during enclosure discovery",
               &MH_NOT_REACHABLE => "Failed to get results, managed host may be down"
              );

sub new 
{
    my $class = shift;
    my $self = {};

    # configuration options
    $self->{CONFIGOPTIONS} = {};
    $self->{PLUGIN} = undef;
    $self->{TECH} = undef;

    bless ($self, $class);
    return $self;
}

sub initialize 
{
    my $self = shift;
    my $tech_type = shift;

    # Based on the vendor information that is passed, 
    #  setup the lower level module.
    my $module_name = "VRTS::Enclosure::Discover::".$tech_type;
    if (eval "require $module_name")
    {
        my $tech_handle = &{\&{"${module_name}::initialize"}};
        $self->{PLUGIN} = $tech_handle;
        $self->{TECH} = $tech_type;
    }
    else
    {
        croak "$@ Technology $tech_type not supported\n";
    }
}

sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    if (not defined $self->{PLUGIN})
    {
        print STDERR "Plugin not loaded\n";
        return undef;
    }

    $self->{PLUGIN}->{"probe_timeout"} = 240;
    my ($op, $ret_code, $err_msg);

    ($op, $ret_code, $err_msg) = $self->{PLUGIN}->get_enclosure_listing($params);
    $err_msg = $err_resource_map{$ret_code} if (not defined $err_msg);

    $ret_code = &PROBE_SUCCESS if (not defined $ret_code);
    return ($op, $ret_code, $err_msg);
}

sub refresh_device_manager
{
    my $self = shift;
    my $params = shift;

    if (not defined $self->{PLUGIN})
    {
        print STDERR "Plugin not loaded\n";
        return undef;
    }

    return $self->{PLUGIN}->refresh_device_manager($params);
}

sub set_discovery_state
{
    my $configid = shift;
    my $arrayid = shift;
    my $state = shift;

    my $app_dir = VRTS::Paths::get_path("AppDir");

    my $array_conf =  File::Spec->catfile($app_dir, "arrayconfig.sql");
    my $array_sql = "UPDATE HABDBSYNC.P_ARRAY_CONFIG_ARRAY_MAPPING SET DISCOVERY_STATE=$state WHERE ID='$configid' and VENDOR_ID='$arrayid';\n";
    open (SQL, "> $array_conf");
    print SQL $array_sql;
    close(SQL);
    
    VRTS::Enclosure::Utils->exec_sql_file($array_conf) if (-f $array_conf);
    unlink ($array_conf) if (-f $array_conf); 

    return 0;
}

sub update_config_mapping
{
    my $agent = shift;

    my $context = $agent->get_context();
    my $discovery_state = &DISCOVERY_FINISHED;

    if (not defined $context->{enclosure_vdid} or
             $context->{enclosure_vdid} eq "")
    {
        # something went wrong during discovery as enclosure_vdid is not present in context
        VRTS::Logger::log('error', 'Discovery failed - enclosure vdid not found');
        $discovery_state = &DISCOVERY_FAILED;

        if (not defined $context->{FAULT_ALREADY_RAISED} or $context->{FAULT_ALREADY_RAISED} != 1)
        {
            VRTS::Enclosure::Utils::raise_discovery_fault($agent, "Enclosure VDID not found");
        }

        # set the task state to failed, and raise a discovery fault
        VRTS::Enclosure::Utils::set_refresh_task_state_failed($agent);

    } else {
        my $array_tb = $agent->gettable("P_ARRAY_SUBSYSTEM");

        my $array_row = $array_tb->getrowwithvalue(&VRTS::Constants::P_ARRAY_SUBSYSTEM::ID,
            $context->{enclosure_vdid});

        $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::TIME] = time();

        $array_tb->addrow($array_row);
        #$array_tb->print_table();
    }

    # Now we also need to report into array config mapping table as well
    my $config_tb = $agent->gettable("P_ARRAY_CONFIG_ARRAY_MAPPING");
    my @config_row;
    $config_row[&VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING::ID] = $context->{"configid"};
    $config_row[&VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING::VENDOR_ID] = $context->{"arrayid"};
    $config_row[&VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING::DISCOVERY_STATE] = $discovery_state;
    $config_row[&VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING::TIME] = time();
    $config_tb->addrow(\@config_row);

    my $conf_tb = $agent->gettable("P_ARRAY_CONFIG");
    my @conf_row;
    $conf_row[&VRTS::Constants::P_ARRAY_CONFIG::ID] = $context->{"configid"};
    $conf_row[&VRTS::Constants::P_ARRAY_CONFIG::FLAG] = 0;
    $conf_row[&VRTS::Constants::P_ARRAY_CONFIG::TIME] = time();
    $conf_tb->addrow(\@conf_row);

    return 0;
}

sub provide_capabilities {
    my $self = shift;

    if (not defined $self->{PLUGIN})
    {
        print STDERR "Plugin not loaded\n";
        return undef;
    }

    my $caps;
    eval {
        $caps = $self->{PLUGIN}->provide_capabilities();
        1;
    } or do {
        print STDERR "Plugin does not implement provide_capabilities\n";
        return undef;
    };
    return $caps;
}
1;
