#$Id: 3PAR.pm,v 1.9 2014/03/25 10:16:42 myadav Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover::3PAR;
use strict;
use Carp;
require Exporter;
require XML::Simple;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;
require VRTS::Paths;
require VRTS::Util;

use File::Spec;
use Data::Dumper;
use JSON;

my ($xml, $vdid_obj);

my %capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 1, "SHOW_RG_PDEV" => 1, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1, "CONSIDER_SEGMENT_LUN" => 1,
   "SHOW_REPLICATION" => 1, "SHOW_TP_SOURCELDEV" => 0, "SHOW_TP_THINLDEV" => 1, "SHOW_LDEV_PDEV" => 1, "LDEV_PDEV_MAP" => 1,
   "PRIMARY_REMOTE_REP_CAPABLE" => 1, "SHOW_VSET" => 0, "SHOW_TP" => 1, "SHOW_RG" => 0, "SECONDARY_REMOTE_REP_CAPABLE" => 1,
   "SHOW_PDEV_FOR_THIN_LDEV" => 1, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0, "SHOW_DEVICE_GROUP_REPLICATION" => 0);



sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::3PAR;
    return $module;

}

sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    my $ch = $params->{"control-host"};
    my $server = $params->{"server"};
    my $port = $params->{"port"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};

    $pass = quotemeta($params->{'pass'});

    my $probe_timeout = $self->{"probe_timeout"};

    # Due to change to halssh to work in a non-interactive mode, 3PAR discovery will only work
    #  with managed host version 5.0 and above, we need to check this as part of probe and provide the correct error code.
    my $xprtlc = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "xprtlc");

    my $my_grep = "grep";
    $my_grep = "findstr" if ($^O =~ /Win32/i);
    my $op = undef;
    my $xprtlc_cmd = VRTS::Util::make_command($xprtlc, "-l", "https://$ch/world/getvitals");
    my $xprtlout = `$xprtlc_cmd`;
    chomp $xprtlout;

    my $jsonobject = new JSON ();
    eval {
      $op = $jsonobject->jsonToObj($xprtlout);
    };

    if ($@)
    {
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    if (not defined $op->{XPRTLD_VERSION})
    {
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    if ($op->{XPRTLD_VERSION} =~ /^[5-9]\./)
    {
        VRTS::Logger::log('debug', "Managed host version is 5.0 or above, supported version");
    }
    else
    {
        VRTS::Logger::log('error', "Managed host version is less than 5.x, not supported");
        return undef, &VRTS::Enclosure::Discover::PLATFORM_UNSUPPORTED;
    }

    # First check if we support the firmware version which should be greater than 2.2.3
    # This is required as the cli options and printing for this version and must be for versions below is very different.

    my $args = " -ssh -l $user -pw $pass -4 $server showversion";
    my $vardir= VRTS::Paths::get_path("AppDir");
    my $script = "$vardir/3par.pl";

    _create_probe_script($args, $script);

    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/3par.pl", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    unlink $script;

    my $blob = $out->[1];
    my $not_supported_fmware = 1;

    my @arr_list = split /\n/, $blob;
    foreach my $line (@arr_list)
    {
        if ($line =~ /Release version/)
        {
            my $version = $line;
            VRTS::Logger::log('debug', "Firmware version detected as [$line]");
            $line =~ s/Release version\s+([0-9])\.([0-9])\.([0-9])//;
            if ($1 > 2)
            {
                $not_supported_fmware = 0;
            }
            elsif ($1 == 2 and $2 > 2)
            {
                $not_supported_fmware = 0;
            } 
            elsif ($1 == 2 and $2 <= 2 and $3 > 3)
            {
                $not_supported_fmware = 0;
            }
            else
            {
                $not_supported_fmware = 1;
            }

            last;
        }
    }

    if ($blob =~ /Connection timed out/)
    {
        return undef, &VRTS::Enclosure::Discover::ARRAY_NOT_REACHABLE;
    }
    elsif ($blob =~ /not found/ or $blob =~ /No such file/)
    {
        # SYMCLI path not found.
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_NOT_FOUND;
    }
    elsif ($blob =~ /invalid credentials/ or $blob =~ /Authentication failur/)
    {
        return undef, &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }
    elsif ($blob =~ /Too many CLI connections/ or $blob =~ /No route to host/ or $blob =~ /Host does not exist/)
    {
        return undef, &VRTS::Enclosure::Discover::ARRAY_NOT_REACHABLE;
    }
    elsif ($not_supported_fmware == 1)
    {
        return undef, &VRTS::Enclosure::Discover::ARRAY_DISCOVERY_FM_LOWER_VERSION_ERROR;
    }

    VRTS::Logger::log('debug', 
        "Firmware version on the array side is higher than the minimum requirement, proceeding with probe");

    $vdid_obj = new VRTS::Enclosure::VDID ();
    $vdid_obj->initialize("3PAR", 0);

    $args = " -ssh -l $user -pw $pass -4 $server showsys";
    $vardir= VRTS::Paths::get_path("AppDir");
    $script = "$vardir/3par.pl";

    _create_probe_script($args, $script);

    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/3par.pl", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    unlink $script;

    my $blob = $out->[1];

    if ($blob =~ /not found/ or $blob =~ /No such file/)
    {
        # SYMCLI path not found.
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_NOT_FOUND;
    }

    my $enc_hash = get_enclosure_details($blob);

    unlink $script;
    return $enc_hash;
}

sub refresh_device_manager
{
    my $self = shift;
    my $params = shift;

    my $out;
    my $ch = $params->{"control-host"};
    my $ch1 = $params->{"secondary-ch"};
    my $install_path = $params->{"install-path"};
    my $array_id = $params->{"array-id"};

    return 0;
}

sub get_enclosure_details
{
    my $data = shift;

    my ($system_id, $serial_no, $model, $array_name);
    my @data_arr = split/\n/, $data;

    return undef if (not defined $data_arr[2] or $data_arr[2] eq "");

    my @enc_data = split / /, $data_arr[2];
    $system_id = $enc_data[0];

    # Converting the system id into hex for vdid formulation.
    $system_id = sprintf("%x", $system_id);

    $array_name = $enc_data[1];
    $model = $enc_data[2]. " ". $enc_data[3];
    $serial_no = $enc_data[4];

    my $vdid = $vdid_obj->get_enclosure_vdid("3PARdata", "VV", $system_id);

    my %val_hash;
    $val_hash{VDID} = $vdid;
    $val_hash{serial} = $serial_no;
    $val_hash{model} = $model;
    $val_hash{displayname} = $array_name;
    $val_hash{product} = "3PARdata";
    $val_hash{vendor} = "VV";

    my $op_hash;
    $op_hash->{$vdid} = \%val_hash;
    return $op_hash;
}

sub _create_probe_script
{
    my $args = shift;
    my $script = shift;

    open(S, ">$script");
    binmode S;

    print S "\#\!\/opt\/VRTSsfmh\/bin\/perl\n\n";

    print S "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) }\n\n";

    print S "require VRTS\:\:Paths\;\n";
    print S "require File\:\:Spec\;\n\n";

    print S "my \$cmd = VRTS\:\:Paths\:\:get_path(\"MHBin\")\;\n";
    print S "\$cmd = File\:\:Spec\-\>catfile(\$cmd, \"halssh\")\;\n";
    print S "if ( \$\^O !~ /Win32/i )\n {\n";
    print S "\$ENV{'LD_LIBRARY_PATH'} = \"/opt/VRTSsfmh/lib\";\n";
    print S "print \"not found\" if (\! -x \"\$cmd\")\;\n";
    print S "system(\"\$cmd $args\")\;\n}\n";
    print S "else\n {\n";
    print S "\$cmd = \$cmd\.\"\.exe\"\;\n";
    print S "print \"not found\" if (\! -x \"\$cmd\")\;\n";
    print S "system(\" \\\"\$cmd\\\" $args\")\;\n}\n";
    close(S);
}

sub provide_capabilities
{
    my $self = shift;

    return \%capabilities;
}

1;
