#$Id: DSCLI.pm,v 1.3 2013/06/24 10:15:39 dkotwal Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $
#$Source: /project/opm-addons/arraymod/src/lib/perl/modules/VRTS/Enclosure/Discover/common/DSCLI.pm,v $

package VRTS::Enclosure::Discover::DSCLI;
use strict;
use Switch;
require XML::SAX::ExpatXS;
require VRTS::XMLHandler;
require VRTS::Enclosure::VDID;

use constant DSCLI_WIN_PATH => "C:\Program Files\IBM\dscli";
use constant DSCLI_UNIX_PATH => "/opt/ibm/dscli";

use constant DS8000_MIN_DSCLI_VER => "5.2.400.426";
use constant DS8000_MIN_FIRMWARE_VER => "5.2.400.426";
use constant DS6000_MIN_DSCLI_VER => "5.2.2.572";
use constant DS6000_MIN_FIRMWARE_VER => "5.2.2.572";

my %model_map = (
        "921" => "8100",
        "931" => "8100",
        "922" => "8300",
        "932" => "8300",
        "9A2" => "8300",
        "9B2" => "8300",
        "941" => "8700",
        "94E" => "8700",
        "951" => "8800",
        "95E" => "8800",
        "511" => "6800",
        "522" => "6800",
        "EX1" => "6800",
        "EX2" => "6800"
);

my $vardir= VRTS::Paths::get_path("AppDir");
my $cli_path = undef;
my $xml;
my %enc_hash;
my $enc_map;

sub new
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;

    my $module = new VRTS::Enclosure::Discover::DSCLI;
    return $module;
}

sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;

    my $cmd;
    my $ch = $params->{"control-host"};
    my $pwdfile = $params->{"ARRAY_DSCLI_PASSWDFILE_PATH"};
    my $pass = quotemeta($params->{'pass'});
    my $out;
    my $cmd_prefix;

    $cli_path = $params->{"install-path"};
    if (not defined $cli_path or $cli_path eq "") {
        if ($^O =~ /Win32/i) {
            $cli_path = DSCLI_WIN_PATH;
        } else {
            $cli_path = DSCLI_UNIX_PATH;
        }
    }

    my $probe_timeout = $self->{"probe_timeout"};

    my $script = "$vardir/dscli_"."$$".".pl";

    if ((defined $pass) and ($pass ne "")) {
        $cmd_prefix = "-hmc1 $params->{'server'} -user $params->{'user'} -passwd $pass ";
    } elsif ((defined $pwdfile) and ($pwdfile ne "")) {
        $pwdfile =~ s/\\/\\\\/g;
        $pwdfile = "\\\""."$pwdfile"."\\\"";
        $cmd_prefix = "-hmc1 $params->{'server'} -user $params->{'user'} -pwfile $pwdfile ";
    } else {
        # Return saying can't have both undefined
        return undef, &VRTS::Enclosure::Discover::DSCLI_PASSWD_PASSWDFILE_NOT_DEFINED;
    }

    # Get enclosure details
    $cmd = $cmd_prefix."lssi -l -fmt xml -bnr off";
    _create_probe_script($cmd, $script);
  
    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/dscli.pl", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
        print "Error in push_exec\n";
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    unlink($script);

    if ($out->[1] =~ /(CMUC\d+E|CMUD\d+E|CMUN\d+E)/i)
    {
        switch ($1) {
            case "CMUC00183E" {
                        # Invalid management console IP address
                        # UI will display - "Unable to connect to Enclosure using the IP Address"
                        return undef,&VRTS::Enclosure::Discover::NO_CONNECTION_INVALID_IP;
            }
            case "CMUC00190E" {
                        # Authentication failure: invalid password
                        # UI will display - "Invalid password"
                        return undef,&VRTS::Enclosure::Discover::INCORRECT_PASSWORD;
            }
            case "CMUC00191E" {
                        # Authentication failure: invalid user name
                        # UI will display - "User does not exist"
                        return undef,&VRTS::Enclosure::Discover::USER_NAME_DOES_NOT_EXIST;
            }
            case "CMUN00018E" {
                        # Unable to connect to the storage management console server
                        # UI will display - "Unable to connect to Enclosure using the IP Address"
                        return undef,&VRTS::Enclosure::Discover::NO_CONNECTION_INVALID_IP;
            }
            case "CMUC00187E" {
                        # CMUC00187E Unable to access CLI user password file
                        # UI will display - "Password File not found at given location
                        return undef,&VRTS::Enclosure::Discover::DSCLI_PASSWDFILE_NOT_FOUND;
            }
            case "CMUC00386E" {
                        # CMUC00386E: The record 10.182.79.201/ccsuser does not exist in the password file
                        # UI will display - Storage Array IP - User Name record does not exist in password file
                        return undef,&VRTS::Enclosure::Discover::DSCLI_PASSWDFILE_RECORD_NOT_FOUND;
            }
            case /(CMUC00201E|CMUN80022E)/ {
                        # "CMUN80022E SOURCE Authentication failure: Your password has expired."

                        # "CMUC00201E VALUE_0: You are logged in with an expired password and limited
                        # authorization. Use the chuser command to change your password and restore normal
                        # authorization."
                        return undef,&VRTS::Enclosure::Discover::ENCLOSURE_PASSWD_EXPIRED;
            }
            else {
                        # Some error
                        return undef,&VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
            }
        }
    }
    elsif (($out->[1] =~ /No such file/i) or ($out->[1] =~ /not found/i))           
    {           
        # DSCLI path not found.         
        # UI will display - "The required vendor components (e.g. CLI) not found at the specified location"         
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_NOT_FOUND;            
    }

    my $handler = VRTS::XMLHandler->new();
    $xml = XML::SAX::ExpatXS->new(Handler => $handler);

    my %markers = ("PROPERTY" => "");
    $handler->initialize(undef, \%markers, "VRTS::Enclosure::Discover::DSCLI::fill_enc_details", undef, 1, undef);
  
    my %parser_args = (Source => {String => $out->[1]});
    eval {$xml->parse(%parser_args)};
    if ($@)
    {
        print "Error in parsing XML\n";
        return undef;
    }  

    # Check the DSCLI and Firmware Versions.
    $cmd = $cmd_prefix . "ver -l -bnr off";
    _create_probe_script($cmd, $script);

    $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "\$VARDIR\/dscli.pl", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
        print "Error in push_exec\n";
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    
    unlink($script);

    #############################################
    # Sample Output -
    #
    # DSCLI          6.6.10.410
    # StorageManager 7.3.4.0.20110224.1
    # ================Version=================
    # Storage Image    LMC
    # =========================
    # IBM.2107-75MA641 5.4.36.5
    #
    #############################################


    my %stgImage_LMC_hash;
    my $dscli_ver;
    my @outArr = split (/\n/,$out->[1]);
    foreach my $line (@outArr) {
        if ($line =~ /DSCLI\s+(\S+)/) {
            $dscli_ver = $1;
        } elsif ($line =~ /(IBM\.\d+-\w+)\s+(\S+)/) {
           $stgImage_LMC_hash{$1} = $2; 
        }
    }

    # Now check that DSCLI and LMC versions are correct for each of the storage images
    while (my ($stgImage_vdid, $stgImage_hash) = each(%{$enc_map})) {
        my $stgImage_id = $stgImage_hash->{displayname};
        # Check version for IBM DS 8000 series models - 921,922,931,932,9A2,9B2,92E,9AE,941,94E,951,95E
        if ($stgImage_hash->{model} =~ /^9\w+/) { # DS8000 series models
            if ((versionCmp($dscli_ver,DS8000_MIN_DSCLI_VER) == 1) or 
                            (versionCmp($stgImage_LMC_hash{$stgImage_id},DS8000_MIN_FIRMWARE_VER) == 1)) {
                return undef,&VRTS::Enclosure::Discover::VENDOR_COMP_VERSION_INCOMPAT;
            }
        } elsif ($stgImage_hash->{model} =~ /[511|521|EX1|EX2]/i) { # DS6000 series models
            if ((versionCmp($dscli_ver,DS6000_MIN_DSCLI_VER) == 1) or 
                            (versionCmp($stgImage_LMC_hash{$stgImage_id},DS6000_MIN_FIRMWARE_VER) == 1)) {
                return undef,&VRTS::Enclosure::Discover::VENDOR_COMP_VERSION_INCOMPAT;
            }
        }
    }

    return $enc_map; 
}

sub fill_enc_details
{
    my $handler = shift;
    my $token = shift;
    my $depth = shift; 

    return if (not defined $handler->{DATA}->{PROPERTY}->{VALUE});

    if ($handler->{DATA}->{PROPERTY}->{NAME} eq "model")
    {
        $enc_hash{model} = $handler->{DATA}->{PROPERTY}->{VALUE};
        my $model = uc($enc_hash{model});
        if (defined($model_map{$model})) {
            $enc_hash{product} = "DS".$model_map{$model};
        }
    }
    elsif ($handler->{DATA}->{PROPERTY}->{NAME} eq "ess_id")
    {
        my $strImageId = $handler->{DATA}->{PROPERTY}->{VALUE};

        #get vdid for storage image
        my $vdid_obj = new VRTS::Enclosure::VDID();
        $vdid_obj->initialize("Dscli",0);
        my $vdid = $vdid_obj->get_enclosure_vdid($strImageId);

        $enc_hash{VDID} = $vdid;
        if ($vdid =~ /-(\S+)_/) {
            # If vdid is "IBM.2107-7516831_900", take 7516831 as the serial number
            $enc_hash{serial} = $1;
        } else {
            $enc_hash{serial} = $vdid;
        }
        $enc_hash{displayname} = $handler->{DATA}->{PROPERTY}->{VALUE};
        $enc_hash{vendor} = "IBM";
        $enc_hash{product} = "DS";
    
        $enc_map->{$vdid} =  \%enc_hash; 
    }
}

# Compares two versions.
# If version1 < version2 return 1
# If version1 > verison2 return -1
# If version1 = version2 return 0
sub versionCmp {
    my $ver1 = shift;
    my $ver2 = shift;

    my @ver1_arr = split(/\./,$ver1);
    my @ver2_arr = split(/\./,$ver2);
    my $ver1_cnt = scalar(@ver1_arr);
    my $ver2_cnt = scalar(@ver2_arr);
    my $ret = 0;

    my $count = 0;

    if ($ver1_cnt < $ver2_cnt) {
        $count = $ver1_cnt;
        $ret = 1;
    } else {
        $count = $ver2_cnt;
        $ret = -1;
    }

    for (my $i = 0; $i < $count; $i++) {
        if ($ver1_arr[$i] < $ver2_arr[$i]) {
            return 1;
        } elsif ($ver1_arr[$i] > $ver2_arr[$i]) {
            return -1;
        }
    }

    if ($ver1_cnt == $ver2_cnt) {
        return 0;
    }

    return $ret;
}

# Create script for dscli command
sub _create_probe_script {
    my $args = shift;
    my $script = shift;
    my $unix_cli = "$cli_path/dscli";
    my $win_cli_dir = $cli_path;
    $win_cli_dir =~ s/^\s*//;
    $win_cli_dir =~ s/\s*$//;
    $win_cli_dir =~ s/\\/\\\\/g;
    my $win_cli = "$win_cli_dir\\\\dscli.exe";
    my $win_dscli_install = "$win_cli_dir\\\\lib";

    $unix_cli =~ s/^\s*//;
    $unix_cli =~ s/\s*$//;

    my $script_fh = FileHandle->new("> $script");
    if (defined($script_fh)) {
        binmode $script_fh;
        print $script_fh "\#\!\/opt\/VRTSsfmh\/bin\/perl\n";
        print $script_fh "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) }\n\n";

        print $script_fh "if ( \$\^O !~ /Win32/i ) {\n";
        print $script_fh "print \"not found\" if (\! -x \"$unix_cli\")\;\n";
        print $script_fh "\t system(\"\\\"$unix_cli\\\" $args\")\;\n}\n";
        print $script_fh "else {\n";
        print $script_fh "\tif ( -e \"$win_dscli_install\") \{\n";
        print $script_fh "\t\t\$ENV\{DSCLI_INSTALL\} = \"$win_dscli_install\"\;\n\t\}\n";
        print $script_fh "print \"not found\" if (\! -x \"$win_cli\")\;\n";
        print $script_fh "\t system(\" \\\"$win_cli\\\" $args\")\;\n}\n";

        $script_fh->close;
    }
}

1;
