#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Discover::SVC;
use strict;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;

my $probe_timeout;

sub new 
{
    my $class = shift;
    my $self = {};
    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;
    my $array_module = new VRTS::Enclosure::Discover::SVC;
    return $array_module;
}

sub provide_capabilities
{
    return undef;
}

##
# Retrieves information about the SVC cluster.
# 
sub get_enclosure_listing
{
    my ($self, $params) = @_;
    my $enc_hash;
    my $cluster_data;
    my $ch = $params->{"control-host"};
    my $cluster_server = $params->{"server"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    my $secfile = $params->{"pem-file"};
    $probe_timeout = $self->{"probe_timeout"};
    
    VRTS::Logger::log ("debug", "get_enclosure_listing(): CH: $ch, Server: $cluster_server, User: $user"); 

    my $ch_version = VRTS::Enclosure::Utils::get_mh_version($ch);
    if (!defined($ch_version)) {
        VRTS::Logger::log ("error", "Discovery host may be down.");
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    } 
    $ch_version =~ /(\d+)\S+/;
    my $main = $1;  
    if ($main < 6) {
        VRTS::Logger::log ("error", "Discovery host not supported. Please specify discovery host having MH Version 6.0 and above");
        return undef, &VRTS::Enclosure::Discover::SVC_MH_VERSION_NOT_SUPPORTED;
    }

    #First try with password. 
    my $pw_opt;
    $pw_opt = "-pw $pass";

    my $out = exec_svc_probe_script($ch, $user, $pw_opt, $cluster_server);
    if ($out->[0] != 0) {
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
    }
    my $err_msg;
    # Parse the command O/P
    if ($out->[1] =~ /Connection timed out|svcinfo(.*)not found|connection refused/is) {
        VRTS::Logger::log ("error", "Wrong IP Address provided or Array is unreachable");
        return undef, &VRTS::Enclosure::Discover::NO_CONNECTION_INVALID_IP;
    } elsif ($out->[1] =~ /Authentication failure, invalid credentials/is) {
        VRTS::Logger::log ("error", "Wrong username or password");
        $err_msg = $out->[1];
    } elsif ($out->[1] =~ /FATAL ERROR:\s*(.*)/is) {
        VRTS::Logger::log ("error", "$1");
        $err_msg = $out->[1];
    } 
    
    #If we are unsuccessful with password then try with ppk file.    
    #If we are successful with password the set the PEMFILE field empty.
    #This is for giving priority for password over ppk file in halssh.
    if (defined($err_msg)) {
        $secfile =~ s/\\/\\\\/g;
        $secfile = "\\\"".$secfile."\\\"";
        $pw_opt = "-i $secfile";
        $out = exec_svc_probe_script($ch, $user, $pw_opt, $cluster_server);
        my $err = $out->[0] >> 8;
        if ($err == 192) {
            VRTS::Logger::log('error', "Request timed out due to no activity for timeout period");
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
        # Parse the command O/P
        if ($out->[1] =~ /CMMVC7016E(.*)/s) {
            if ($err_msg =~ /Authentication failure, invalid credentials/is) {
                return undef, &VRTS::Enclosure::Discover::SVC_WRONG_CREDENTIALS_OR_WRONG_USERNAME_AND_WRONG_SSHFILE;
            } elsif ($err_msg =~ /No supported authentication methods available/is) {     
                VRTS::Logger::log ("error", "$1");
                return undef, &VRTS::Enclosure::Discover::SVC_WRONG_USERNAME_FOR_SSHFILE;
            }
        } elsif (($out->[1] =~ /FATAL ERROR|Server refused our key|Unable to use key file/is) || ($out->[1] eq "")){
            if (($err_msg =~ /No supported authentication methods available/is) && 
                    ($out->[1] =~ /No supported authentication methods available/is)) {
                VRTS::Logger::log ("error", "Wrong username and ssh key file combination provided");
                return undef, &VRTS::Enclosure::Discover::SVC_WRONG_USERNAME_AND_WRONG_SSHFILE;
            } elsif ($err_msg =~ /Authentication failure, invalid credentials/is) {
                return undef, &VRTS::Enclosure::Discover::SVC_WRONG_CREDENTIALS_OR_WRONG_USERNAME_AND_WRONG_SSHFILE;
            } else {
                return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
            }        
        }
    } else {
        $params->{PEMFILE} = undef;
    }         
    
    # Sample O/P:
    # 0000020061005C2A:SANVC1:local:::0000020061005C2A
    my @alines = split(/\n/, $out->[1]);
    foreach (@alines)
    {
        chomp;
        my ($clus_id, $clus_name, $clus_loc, $clus_part, $clus_band, $clus_alias) = split (/:/);
        next if $clus_loc ne "local"; # Look at local clusters only
        if (not defined($clus_id) || $clus_id eq "") {
            VRTS::Logger::log ("error", "get_enclosure_listing(): Failed to get SVC cluster id");
            return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
        }
        VRTS::Logger::log ("debug", "get_enclosure_listing(): Found cluster id: $clus_id, cluster name: $clus_name");
        # Form hash of cluster attributes from the command O/P
        $enc_hash = _get_enclosure_details ($cluster_server, $clus_id);
        last;
    }

    if (not defined($enc_hash)) {
        return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
    }
    else {
        return $enc_hash, &VRTS::Enclosure::Discover::PROBE_SUCCESS;
    }
}

#
#  Creates probe script to be executed on the SVC cluster.
#
sub _create_probe_script
{
    my ($user, $pw_opt, $cluster_server) = @_;
    my $file = File::Spec->catfile(VRTS::Paths::get_path("TempDir"),"SVC.halssh.$$.$cluster_server.pl");
    my $probecmd = "svcinfo lscluster -nohdr -delim :";
    VRTS::Logger::log ("debug","_create_probe_script(): $probecmd");

    my $r = open(S, ">$file");
    if (!$r)
    {
        VRTS::Logger::log ("error", "_create_probe_script(): Failed to open file $file for writing: $!");
        return undef;
    }

    binmode S;
    print S '#!/opt/VRTSsfmh/bin/perl'."\n";
    print S 'BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }'."\n";
    print S 'require VRTS::Util;'."\n";
    print S '$ENV{'."'".'LD_LIBRARY_PATH'."'".'}='."'".'/opt/VRTSsfmh/lib\''." if ( ".'$^O'." !~ /Win32/i );"."\n";
    print S 'my $cmd = File::Spec->catfile(VRTS::Paths::get_path("MHBin"),"halssh");'."\n";
    print S 'if ( $^O =~ /Win32/i )'."\n";
    print S '{'."\n";
    print S '    $cmd="\"".$cmd.".exe\"";'."\n";
    print S '    $cmd = $cmd." -ssh -l '.$user.' '.$pw_opt.' -4 '.$cluster_server.' \"'.$probecmd.'\"";'."\n";
    print S '}'."\n";
    print S 'else'."\n";
    print S '{'."\n";
    if ($pw_opt =~ /^-pw/) {
        print S '    $cmd = $cmd." -ssh -l '.$user.' '.$pw_opt.' -4 '.$cluster_server.' \"'.$probecmd.'\"";'."\n";
    } else {
        print S '    $cmd = $cmd." -ssh -batch -l '.$user.' '.$pw_opt.' -4 '.$cluster_server.' \"'.$probecmd.'\"";'."\n";
    }
    print S '}'."\n";
    print S 'system("$cmd");'."\n";
    print S 'exit(0);'."\n";
    close(S);

    return $file;
}

##
#  Returns a hash of enclosure details.
#
sub _get_enclosure_details
{
    my ($cluster_server, $clus_id) = @_;
    my $vdid_obj = new VRTS::Enclosure::VDID ();
    my $array_vendor = "IBM";
    # The machine type is always 2145 for SVC. We use the same model number for V7000 as well.
    my $array_model = "2145";
    my $array_family = "SVC";
    $vdid_obj->initialize($array_family, 0);
    my $enc_hash;
    my $enc_detail = {};

    $clus_id = lc ($clus_id);
    my $e_vdid = $vdid_obj->get_enclosure_vdid($array_vendor, $array_model, $clus_id);
    $enc_detail->{VDID} = $e_vdid;
    $enc_detail->{serial} = $clus_id;
    $enc_detail->{displayname} = $array_family.'@'.$cluster_server;
    $enc_detail->{id} = $e_vdid;
    $enc_detail->{product} = $array_family;
    $enc_detail->{vendor} = $array_vendor;
    $enc_detail->{model} = $array_model;
    $enc_hash->{$e_vdid} = $enc_detail;

    return $enc_hash;
}

# This method creates the wrapper perl script to be run on a control host for 
# SVC discovery. The script contains code to run the SVC commands over ssh
# on the remote SVC cluster host.
sub create_discovery_script
{
    my ($cluster, $user, $pass, $secfile, $g_discover_shell_script)  = @_;

    my $perl_script = File::Spec->catfile(VRTS::Paths::get_path("TempDir"), "SVC_halssh_".$$."_$cluster.pl");
    my $PERLSC = open(S, "> $perl_script");
    if (! $PERLSC)
    {
        VRTS::Logger::log ("error", "_create_discovery_script(): Failed to open file $perl_script for writing: $!");
        return undef;
    }

    # The script shall be constructed in 4 parts:
    # (1) The initial code containing module includes, ssh script file creation, and beginning of ssh script 
    # variable definition
    # (2) The ssh script itself, as defined in $g_discover_shell_script variable.
    # (3) The end of ssh script section, and variable initializations, which shall require expansion of variables
    # passed to this function
    # (4) The final code which includes invoking the script via halssh module.

    my $perl_script1 = <<'PERLSC1';
#!/opt/VRTSsfmh/bin/perl
BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }

require VRTS::Paths;
require File::Spec;

my $g_discover_shell_script = <<'SHELLSC';
PERLSC1

# The shell script is appended to the above code - see the end part of this method.

    my $perl_script2 = <<PERLSC2;
SHELLSC

my (\$cluster, \$user, \$pass, \$secfile)  = (\'$cluster\', \'$user\', \'$pass\', \'$secfile\');
PERLSC2

    my $perl_script3 =<<'PERLSC3';

my $cmd = VRTS::Paths::get_path("MHBin");
$cmd = File::Spec->catfile($cmd, "halssh");

my $ssh_script = File::Spec->catfile(VRTS::Paths::get_path("TempDir"), "SVC_discover_$$_$cluster.sh");
if (!open(S, "> $ssh_script")) {
    return;
}

binmode S;
print S $g_discover_shell_script;
close (S);

my $args = "-ssh -l $user ";
if ($secfile) {
    $args = $args."-i \"$secfile\" "; 
} elsif ($pass) {
    $args = $args."-pw $pass ";
} 
$args = $args."-4 $cluster -m $ssh_script";

if ($^O !~ /Win32/i ) {
    $ENV{'LD_LIBRARY_PATH'} = '/opt/VRTSsfmh/lib';
    print "not found" if (! -x "$cmd");
    system("$cmd $args");
} else {
    $cmd = $cmd.".exe";
    print "not found" if (! -x "$cmd");
    system(" \"$cmd\" $args");
}
        
PERLSC3

    my $script = $perl_script1 . $g_discover_shell_script . $perl_script2 . $perl_script3;
    binmode S;
    print S $script; 
    close (S);
    return $perl_script;
}

sub exec_svc_probe_script
{
    my ($ch, $user, $pw_opt, $cluster_server) = @_;
    # Create script for probing the SVC cluster.
    my $script = _create_probe_script($user, $pw_opt, $cluster_server);
    if (not defined ($script)) {
        VRTS::Logger::log ("error", "Error creating script");
        return undef, &VRTS::Enclosure::Discover::PROBE_INTERNAL_ERROR;
    }

    # Run the script over ssh on the SVC cluster
    my $out = VRTS::Enclosure::Utils::push_exec ($ch, $script, '$VARDIR/SVC.halssh.'."$$.$cluster_server.pl", $probe_timeout);
    unlink ($script);
    return $out;
}
1;
