#$Id: Stats.pm,v 1.3 2013/07/22 11:29:21 myadav Exp $
#$Copyright: Copyright (c) 2014 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $

package VRTS::Enclosure::Stats;

use strict;
use warnings;

use Carp;
require Exporter;
use File::Spec;

require JSON;

require VRTS::Statlog;

#Import constants for individual table definitions 
require VRTS::Constants::P_ARRAY_DEVICE_GROUP;

use vars qw($VERSION @ISA @EXPORT);

@ISA = qw(Exporter);
@EXPORT = qw(store_stats);

$VERSION = '5.0';

my $trending = {};

#Initialize trending structure in following format
#{
#   TBL_NAME => 'Name of the table',
#   COLUMNS => [List of columns],
#   STAT_DB_LOG_NAME => 'Name of statlog',
#   KEY => {
#       'statlog segment prefix' => {
#           COLUMN => 'column name',
#           SUFFIX => 'comma seperated list of suffix for segment name',
#           FUNCTION => 'optional function name for transforming data'
#           WHERE => {
#               FUNCTION => 'fuction name to filter data',
#               PARAMS => 'parameters to be passed to the function' 
#
#           }
#        }
#   }
#}
$trending->{TP} = {
    TBL_NAME => 'P_ARRAY_DEVICE_GROUP',
    COLUMNS => ['ID', 'VVOL_SIZE', 'FREE_SIZE', 'TOTAL_SIZE', 'TYPE'],
    STAT_DB_LOG_NAME => '1day.array_tp_stats',
    KEY => {
        'sc' => {
            COLUMN => 'VVOL_SIZE',
            SUFFIX => 'ID',
            WHERE => {
                FUNCTION => 'equals',
                PARAMS => 'TYPE,"TP"'
            }
        },
        'uc' => {
            COLUMN => 'TOTAL_SIZE, FREE_SIZE',
            SUFFIX => 'ID',
            FUNCTION => 'subtract',
            WHERE => {
                FUNCTION => 'equals',
                PARAMS => 'TYPE,"TP"'
            }
        }
    }
};

#########################################################################
# Description: Utility method to perform substract operation
# Input :
#	Two or more numbers
# Output: 
# 	Difference of this numbers
#########################################################################
sub subtract {
    my @params = @_;
    my $ret = shift @params;
    $ret -= $_ for (@params);
    return $ret;
}

#########################################################################
# Description: Utility method to check equality of two operators
# Input :
#	Two strings
# Output: 
#   1 if strings are equal, zero otherwise	
#########################################################################
sub equals {
    my $lval = shift;
    my $rval = shift;
    if($lval eq $rval) {
      return 1; 
    }
    return 0;
}

#########################################################################
# Description: Parse and store data into statlog. This method relies on
#              hash "trending" to determine which all parameters should
#              be extracted
# Input :
#   $agent - Agent object
# Output: 
# 	None	
#########################################################################
sub store_stats {
    my $agent = shift;

    my $method = $agent->{NAME};
    my $context = $agent->get_context();
    my $stat_hash;
    my $stat_val;
    my $stat_key;
    my $statlog;
    my $ret;

    $statlog = new VRTS::Statlog();

    #Parse information based on each trending structure 
    foreach my $k (keys(%{$trending})) {
        my $tblname = $trending->{$k}->{TBL_NAME};
        my $cols_hash;
        my @vals = ();
        my $cab_tb = $agent->gettable($tblname);
        my $each_current = $cab_tb->each_current();
        if (scalar(@{$each_current}) == 0) {
            $each_current = $cab_tb->each_reported();
        }

        #Initialize statlog
        $statlog->initialize({loglevel => 3, 
                logname => $trending->{$k}->{STAT_DB_LOG_NAME}});

        #Iterate through each row and extract required data
        foreach my $row (@{$each_current}) {
            my $cols = $trending->{$k}->{COLUMNS};

            #Copy data for a row into cols_hash
            foreach my $col (@$cols) {
                my $x = &{\&{"VRTS::Constants::".$tblname."::".$col}};
                $cols_hash->{$col}= $row->[1]->[$x];
            }

            #next if(is_filtered($cols_hash, $trending->{$k}->{WHERE}));

            #Extract and transform data and store in statlog
            foreach my $key (keys(%{$trending->{$k}->{KEY}})) {

                #Filter data based on where clause
                if(defined($trending->{$k}->{KEY}->{$key}->{WHERE})) {
                    my $params = $trending->{$k}->{KEY}->{$key}->{WHERE}->{PARAMS};
                    my $filter_fn = $trending->{$k}->{KEY}->{$key}->{WHERE}->{FUNCTION};
                    next if(!defined($params) || !defined($filter_fn));
                    my(@parr) = split(/,/, $params);
                    #Substitute parameter id with column values
                    for (my $i = 0; $i < scalar(@parr); $i++) {
                        if(defined($cols_hash->{$parr[$i]})) {
                            $parr[$i] = $cols_hash->{$parr[$i]};
                        }
                        $parr[$i] =~ s/\"//g;
                    }
                    #Call filter function
                    $ret = &{\&{$filter_fn}}(@parr);
                    #Skip the row if data is filtered. Function will return 0 if filtered
                    next if(!$ret);
                }

                my $trend = $trending->{$k}->{KEY}->{$key};
                $trend->{SUFFIX} =~ s/\s+//g;

                #Split keys
                my @keys;
                @keys= split(',', $trend->{SUFFIX});

                #Replace keys with row values and push
                my @rowdata;
                push @rowdata, $cols_hash->{$_} for(@keys);

                #Construct segment name for statlog
                $stat_key = join('.', @rowdata);
                $stat_key = $key.".".$stat_key;

                if($trend->{FUNCTION}) {
                    #Transform data if a function is provided
                    $stat_val = 'NOT_DEFINED';
                    $trend->{COLUMN} =~ s/\s+//g;
                    @keys = split(',', $trend->{COLUMN});
                    @rowdata= ();
                    push @rowdata, $cols_hash->{$_} for(@keys);

                    #Call function and store result as stat_val
                    $stat_val = &{\&{$trend->{FUNCTION}}}(@rowdata);
                }
                else {
                    #No data transformation required. Store data in stat_val
                    $stat_val = $cols_hash->{$trend->{COLUMN}}
                }

                #Insert data in statlog
                if($stat_val =~ /^\d*\.?\d+$/) {
                    my $v;
                    $v->{key} = $stat_key;
                    $v->{val} = $stat_val;
                    push @vals, $v;
                    $statlog->insert({}, \@vals);
                    @vals = ()
                }
            }
        }
    }

}

1;
